from vcCommand import *

app = getApplication()
cmd = getCommand()


def OnStart():
  global p_only_selected, p_set_ptp, p_set_lin, p_joint_force, p_joint_speed, p_cartesian_acc, p_cartesian_speed, p_set
  
  p_only_selected = cmd.getProperty('Only Selected Robots')
  p_set_ptp = cmd.getProperty('Set PTP speeds')
  p_set_lin = cmd.getProperty('Set LIN speeds')
  p_joint_force = cmd.getProperty('JointForce')
  p_joint_speed = cmd.getProperty('JointSpeed')
  p_cartesian_acc = cmd.getProperty('CartesianAcc')
  p_cartesian_speed = cmd.getProperty('CartesianSpeed')
  p_set = cmd.getProperty('Set')
  
  if not p_only_selected:
    p_only_selected = cmd.createProperty(VC_BOOLEAN, 'Only Selected Robots')
    p_only_selected.Value = False
    
    p_set_ptp = cmd.createProperty(VC_BOOLEAN, 'Set PTP speeds')
    p_set_ptp.Value = True
    
    p_set_lin = cmd.createProperty(VC_BOOLEAN, 'Set LIN speeds')
    p_set_lin.Value = True
    
    p_joint_force = cmd.createProperty(VC_REAL, 'JointForce')
    p_joint_force.Quantity = 'Percentage'
    p_joint_force.Value = 1.00
    
    p_joint_speed = cmd.createProperty(VC_REAL, 'JointSpeed')
    p_joint_speed.Quantity = 'Percentage'
    p_joint_speed.Value = 1.00
    
    p_cartesian_acc = cmd.createProperty(VC_REAL, 'CartesianAcc')
    p_cartesian_acc.Quantity = 'Acceleration'
    p_cartesian_acc.Value = 2000.0
    
    p_cartesian_speed = cmd.createProperty(VC_REAL, 'CartesianSpeed')
    p_cartesian_speed.Quantity = 'Speed'
    p_cartesian_speed.Value = 1000.0
    
    p_set = cmd.createProperty(VC_BUTTON, 'Set')
  
  p_set.OnChanged = set
  p_set_ptp.OnChanged = setup
  p_set_lin.OnChanged = setup
  
  executeInActionPanel() 


def setup(*args):
  p_joint_force.IsVisible = p_set_ptp.Value
  p_joint_speed.IsVisible = p_set_ptp.Value
  p_cartesian_acc.IsVisible = p_set_lin.Value
  p_cartesian_speed.IsVisible = p_set_lin.Value


def set(*args):
  if p_only_selected.Value:
    comps = app.SelectionManager.getSelection(VC_SELECTION_COMPONENT)
    if not comps and app.TeachContext.ActiveRobot:
      comps = [app.TeachContext.ActiveRobot]
  else:
    comps = app.Components
  
  for comp in comps:
    #Robot programs
    exes = comp.findBehavioursByType(VC_ROBOTEXECUTOR)
    for exe in exes:
      rous = exe.Program.Routines
      rous.insert(0, exe.Program.MainRoutine)
      for rou in rous:
        for stmt in rou.Statements:
          set_statement(stmt)
    
    #Transport controllers
    try:
      if p_set_ptp.Value:
        comp.getProperty('Speeds::JointForce').Value = p_joint_force.Value
        comp.getProperty('Speeds::JointSpeed').Value = p_joint_speed.Value
      if p_set_lin.Value:
        comp.getProperty('Speeds::CartesianAcc').Value = p_cartesian_acc.Value
        comp.getProperty('Speeds::CartesianSpeed').Value = p_cartesian_speed.Value
    except:
      pass


def set_statement(stmt):
  if stmt.Type == VC_STATEMENT_PTPMOTION:
    if p_set_ptp.Value:
      stmt.JointForce = p_joint_force.Value
      stmt.JointSpeed = p_joint_speed.Value
  elif stmt.Type == VC_STATEMENT_LINMOTION:
    if p_set_lin.Value:
      stmt.Acceleration = p_cartesian_acc.Value
      stmt.Deceleration = p_cartesian_acc.Value
      stmt.MaxSpeed = p_cartesian_speed.Value
  elif stmt.Type == VC_STATEMENT_IF:
    for cstmt in stmt.ThenScope:
      set_statement(cstmt)
    for cstmt in stmt.ElseScope:
      set_statement(cstmt)
  elif stmt.Type == VC_STATEMENT_WHILE:
    for cstmt in stmt.Scope:
      set_statement(cstmt)